// Bibliotheque vkey pour gérer un clavier Sparkfunk
// Vkey Voltage Keyboard
// qui permet sur une seule entrée annalogique d'avoir 12 touches 'tout ou rien'
// Bibliotheque ecrite par Cordier Yves le 18/03/2017
// Libre de droit
// V1.0 le 18/03/2017
// V1.1 le 06/12/2017 ajout de getDigit

// entete de Vkey.h
/*
Cette librairie permet de gérer un clavier numerique 12 touches à sortie analogique
produit par sparkfun

suivant la touche enfoncée, et suivant la tension d'alimentation, la tension
de sortie du clavier varie comme suit


N°		Vout (5V)	Vout(3.3V)
Aucune 	.057 		.053
12 		.198 		.194
11 		.396 		.388
10 		.596 		.583
9 		.794 		.776
8 		.992 		.970
7 		1.190 		1.164
6 		1.388 		1.358
5 		1.585 		1.551
4 		1.781 		1.744
3 		1.979 		1.938
2 		2.176 		2.131
1 		2.372 		2.323

apres conversion analogique-numerique par le micro-contrôleur,

les valeurs de sorties s'echelonnent par pas de :
- 42 pour une alimentation 5v
- 59 pour une alimentation 3.3v

aucune touche correspond alors à
- 9 pour une alimentation 5v
- 14 pour une alimentation 3.3v

L'objet de type Vkey permet de donner le numéro de la touche opressée
en mettant en place un processus anti-rebond logiciel ainsi qu'une procedure
de répétition à double vitesse dans le cas d'un appuis long.

Les méthodes utilisables sont :
    Vkey(int pin = A0) 	Le constructeur avec comme valeur d'appel par defaut A0
    int getDigitFmt() 	renvoi 0 si aucune touche n'est appuyée, 
						un code de 1 à 12 correspondant à la touche appuyée en front
	int getDigit() 		renvoi 0 si aucune touche n'est appuyée, 
						un code de 1 à 12 correspondant à la touche appuyée					
    setRepetition(long r = 500) le temps de repetition de touches. 
						La répétition lente tous les r millis secondes
						par defaut 500ms
    setMaxRepAvantAcc(int n = 5) le nombre de repetitions avant acceleration 
						La répétition double de fréquence après n répétitions 
						Par défaut 5
    setPasValeur(int p = 42) A n'utiliser que dans le cas ou la reconaissance 
						automatique de la tension d'alimentation du clavier 
						par la fonction begin n'etait pas fiable.
    void begin()		initialisation des paramètres internes
						Appeler cette fonction dans le setup()
						en prenant garde qu'aucunes touche ne soit enfoncée au depart
						sinon travailler avec setPasValeur
    int getDerniereTouche Renvoi le code de la dernière touche enfoncée
 

*/
#ifndef Vkey_H
 
#define Vkey_H

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif
// *************************************************************
// La classe permettant de gérer le clavier numérique annalogique

class Vkey
{ public:
    Vkey(int pin = A0); // constructeur
    int getDigitFmt(); // renvoi 0 si aucune touche n'est appuyée, un code de 1 à 12 correspondant à la touche appuyée
	int getDigit(); // renvoi 0 si aucune touche n'est appuyée, un code de 1 à 12 correspondant à la touche appuyée
    void setRepetition(long r = 500); // le temps de repetition de touches
    void setMaxRepAvantAcc(int n = 5); // le nombre de repetitions avant acceleration 
    void setPasValeur(int p = 42);
    void begin(); // initialisation des paramètres internes
    int getDerniereTouche();
	
  private: // les paramètres internes utilisés par les méthodes
  
    int pinkey ; // la broche annalogique sur laquelle est brancée le clavier
    long tempsDernierAppuis; // instant de validation de la dernière touche enfoncée
    long tempsRepetition; // delais de repetition de touches si appuis prolongé
    int maxRepetitionAvantAcceleration; // nb de repetitions avant acceleration defillement
    int nbRepetition; // nombres de fois ou la touche a été répétée (sert à la double vitesse de répétition (au bout de 5 répétition valeur par defaut))
    long dernierAppel; // pour eviter les rebonds
    int derniereTouche; // derniere touche enfoncée ou 0 si aucune
    int derniereToucheValidee; // derniere touche valide detectée
    int  pasValeur ; // varie suivant les tensions d'alimentation.
};
#endif  // Vkey_H
